package sim;

import java.awt.*;
import java.awt.event.*;

import sim.util.StatusBox;

public class SouthPanel extends Panel
{
	public static StatusBox STATUS_BOX;
	
	public SouthPanel()
	{
		super();

		this.setBackground(MainWindow.BACKGROUND);
		this.setLayout(new BorderLayout());
		SouthPanel.STATUS_BOX = new StatusBox("Initialising...", null, true);
		this.add(SouthPanel.STATUS_BOX);
		
		SouthPanel.STATUS_BOX.addMouseListener(new MouseAdapter()
		{
			public void mouseClicked(MouseEvent e)
			{
				SouthPanel.STATUS_BOX.requestFocus();
			}
		});
	}
}

