package sim.engine;

public class EngineException extends Exception
{
	private EngineModule affected1;
	private EngineModule affected2;
	
	public EngineException(String comment)
	{
		super(comment);
	}
	
	public void setFirstAffected(EnginePeer ep)
	{
		this.affected1 = ep.getParent();
	}
	
	public void setSecondAffected(EnginePeer ep)
	{
		this.affected2 = ep.getParent();
	}
	
	public EngineModule getFirstAffected()
	{
		return this.affected1;
	}
	
	public EngineModule getSecondAffected()
	{
		return this.affected2;
	}
}
