import sim.MainWindow;

/**
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'Class1'
 * created in the main() method.
 */
public class Run
{
	/**
	 * The main entry point for the application. 
	 *
	 * @param args Array of parameters passed to the application
	 * via the command line.
	 */
	
	public static void main (String[] args)
	{
		// TODO: Add initialization code here
		
		MainWindow x = new MainWindow(null);
	}
}
