package sim.util;

import java.awt.*;

public class DialogButton extends Button
{
	public static int HEIGHT = 23;
	public static int WIDTH = 75;
	
	public DialogButton(String s)
	{
		super(s);
		this.setSize(75, 23);
	}
	
	public Dimension getPreferredSize()
	{
		return this.getSize();
	}
	
	public Dimension getMinimumSize()
	{
		return this.getSize();
	}

	public Dimension getMaximumSize()
	{
		return this.getSize();
	}
}