package sim.util;

import java.awt.*;

public class OutlinedPanel extends Container
{
	public void paint(Graphics g)
	{
		super.paint(g);
	
		Dimension d = this.getSize();
		
		g.setColor(Color.white);
		g.drawLine(0, 0, d.width - 2, 0);
		g.drawLine(0, 0, 0, d.height - 2);
		
		g.setColor(Color.black);
		g.drawLine(d.width - 1, 0, d.width - 1, d.height - 1);
		g.drawLine(0, d.height - 1, d.width - 1, d.height - 1);
		
		g.setColor(this.getBackground().darker());
		g.drawLine(d.width - 2, 1, d.width - 2, d.height - 2);
		g.drawLine(1, d.height - 2, d.width - 2, d.height - 2);
	}
}
