package sim.engine;

import sim.Wrapper;

public interface EngineModule
{
	public abstract void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer) throws EngineException;
	public abstract void createEnginePeer(EnginePeerList epl);
	public abstract void reset();
	public abstract Wrapper getParentWrapper();
}