package sim.util;

import java.awt.*;

import sim.MainWindow;

public class SimFileDialog extends FileDialog
{
	public SimFileDialog(Frame f)
	{
		super(f);
	}
	
	public void show()
	{
		this.setLocation(MainWindow.MAIN_WINDOW.getLocation());
		super.show();
	}
}
