package sim;

import java.awt.Color;

public abstract class WrapperPainted extends Wrapper
{
	public static Color BACKGROUND = MainWindow.MAIN_WINDOW.getBackground();
	protected Color brush;

	public void changeColor(Color c)
	{
		this.brush = c;
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	public WrapperPainted()
	{
		super();
		
		this.brush = Color.black;
	}
	
	public void scale()
	{
	}
	
/* ==================================================================
	Rotation abd Flipping Part
	================================================================= */
	public boolean canRotate()
	{
		return false;
	}
	
	public boolean canFlip()
	{
		return false;
	}
	
	public void flipHorizontal()
	{
	}
	
	public void flipVertical()
	{
	}
	
	public void setFlipped(boolean newFlip)
	{
	}
	
	public void setAngle(int newAngle)
	{
	}
	
	public void restoreAngleFlipped(int oldAngle, boolean oldFlip)
	{
	}
	
	public void rotateLeft()
	{
	}
	
	public void rotateRight()
	{
	}
}
