package sim;

import java.awt.*;

import sim.util.SimButton;

public class ModuleButton extends SimButton
{
	private CreationModule structure;
	
	public ModuleButton(String path, String help)
	{
		super(path, help, WestPanel.BUBBLE_WAIT);
		this.structure = null;
	}
	
	public ModuleButton(CreationModule cm)
	{
		super(cm.getIcon(), cm.getBubbleHelp(), WestPanel.BUBBLE_WAIT);
		this.structure = cm;
	}
	
	public CreationModule getStructure()
	{
		return this.structure;
	}
}
