package sim;

import java.awt.Component;

public interface PopupModule
{
	public abstract boolean hasProperties();
	public abstract Component getPropertyWindow();
	public abstract void respondToChanges(Component property);
	
	public abstract int getNumberOfMenuItems();
	public abstract String getMenuItemName(int index);
	public abstract void respondToMenuItem(String itemName);
	
	public abstract void restoreOriginalProperties();
}
