package sim;

import java.util.Vector;

public class WrapperList extends Vector
{
	public WrapperList(int size)
	{
		super(size);
	}
	
	public WrapperList()
	{
		super();
	}
		
	public int getSize()
	{
		return super.size();
	}
	
	public Wrapper getItemAt(int index)
	{		
		return (Wrapper)super.elementAt(index);
	}
	
	public void removeItemAt(int index)
	{
		super.removeElementAt(index);
	}
	
	public void removeItem(Wrapper oldValue)
	{
		super.removeElement(oldValue);
	}
	
	public void removeAll()
	{
		super.removeAllElements();
	}
	
	public void changeItem(int index, Wrapper newValue)
	{
		super.setElementAt(newValue, index);
	}
	
	public void resetItem(int index)
	{
		super.setElementAt(null, index);
	}
	
	public void insertItem(Wrapper newValue)
	{
		super.addElement(newValue);
	}
	
	public void insertItemAt(int index, Wrapper newValue)
	{
		super.insertElementAt(newValue, index);
	}
	
	public boolean contains(Wrapper value)
	{
		return super.contains(value);
	}
}
