package sim;

public interface StorageModule
{
	public abstract int getNumberOfSpecificParameters();
	public abstract String getSpecificParameters();
	public abstract void loadWrapper(String[] specificParameters) throws SimException;
}