package sim;

import java.awt.*;

public interface CreationModule
{
	public abstract Image getIcon();
	public abstract Wrapper createWrapper();
	public abstract Wrapper createWrapper(Point gridPosition);
	public abstract String getBubbleHelp();
}
