package sim.engine;

public class Data
{	
	private boolean value;				// actual data
	private boolean undefined;
	
	public Data(boolean newValue, boolean newUndefined)
	{
		this.value = newValue;
		this.undefined = newUndefined;
	}
	
	public boolean getValue()
	{
		return this.value;
	}
	
	public boolean isUndefined()
	{
		return this.undefined;
	}
}