package sim.util;

import java.awt.*;

public class LightContainer extends Container
{
	public LightContainer()
	{
		super();
	}
	
	public LightContainer(LayoutManager layout)
	{
		super();
		this.setLayout(layout);
	}
	
	public Dimension getPreferredSize()
	{
		return this.getSize();
	}
}
